///////////////////////////////////////////////////////////////////////
// File:  CRC32_D1.v                             
// Date:  Thu Oct 24 02:58:55 2002                                                      
//                                                                     
// Copyright (C) 1999 Easics NV.                 
// This source file may be used and distributed without restriction    
// provided that this copyright statement is not removed from the file 
// and that any derivative work contains the original copyright notice
// and the associated disclaimer.
//
// THIS SOURCE FILE IS PROVIDED "AS IS" AND WITHOUT ANY EXPRESS
// OR IMPLIED WARRANTIES, INCLUDING, WITHOUT LIMITATION, THE IMPLIED
// WARRANTIES OF MERCHANTIBILITY AND FITNESS FOR A PARTICULAR PURPOSE.
//
// Purpose: Verilog module containing a synthesizable CRC function
//   * polynomial: (0 1 2 4 5 7 8 10 11 12 16 22 23 26 32)
//   * data width: 1
//                                                                     
// Info: jand@easics.be (Jan Decaluwe)                           
//       http://www.easics.com                                  
///////////////////////////////////////////////////////////////////////

module pn_gen (clk, reset, pn_out, pn_shift);
// Ports
input  clk, reset, pn_shift;
output[31:0]  pn_out;
  // polynomial: (0 1 2 4 5 7 8 10 11 12 16 22 23 26 32)
  // data width: 1
wire D;
reg [31:0] NewCRC, C;

  assign D = 0;
  assign pn_out = C;

  always@(posedge clk)
  begin
  if (reset)
	C <= 32'h11111111;
  else if (pn_shift)
	C <= NewCRC;
  end

  always@(C or D)
    begin
    NewCRC[0] = D ^ C[31];
    NewCRC[1] = D ^ C[0] ^ C[31];
    NewCRC[2] = D ^ C[1] ^ C[31];
    NewCRC[3] = C[2];
    NewCRC[4] = D ^ C[3] ^ C[31];
    NewCRC[5] = D ^ C[4] ^ C[31];
    NewCRC[6] = C[5];
    NewCRC[7] = D ^ C[6] ^ C[31];
    NewCRC[8] = D ^ C[7] ^ C[31];
    NewCRC[9] = C[8];
    NewCRC[10] = D ^ C[9] ^ C[31];
    NewCRC[11] = D ^ C[10] ^ C[31];
    NewCRC[12] = D ^ C[11] ^ C[31];
    NewCRC[13] = C[12];
    NewCRC[14] = C[13];
    NewCRC[15] = C[14];
    NewCRC[16] = D ^ C[15] ^ C[31];
    NewCRC[17] = C[16];
    NewCRC[18] = C[17];
    NewCRC[19] = C[18];
    NewCRC[20] = C[19];
    NewCRC[21] = C[20];
    NewCRC[22] = D ^ C[21] ^ C[31];
    NewCRC[23] = D ^ C[22] ^ C[31];
    NewCRC[24] = C[23];
    NewCRC[25] = C[24];
    NewCRC[26] = D ^ C[25] ^ C[31];
    NewCRC[27] = C[26];
    NewCRC[28] = C[27];
    NewCRC[29] = C[28];
    NewCRC[30] = C[29];
    NewCRC[31] = C[30];
    end

endmodule




