//Author: Norm
//Modified: LT PANG
//Date: 10/20/02

//`include "constants.v"
//`include "sdram_const.v"

// Testing module for SDRAM.
module sdram_cntrl (	 //RAM_D_IN, 	// to SDRAM
		 //RAM_D_OUT,		// to SDRAM
		 RAM_D_EN,		// to SDRAM
		 RAM_CLK,		// to SDRAM
		 RAM_CLKE,		// to SDRAM
		 RAM_DQMH,		// to SDRAM
		 RAM_DQML,		// to SDRAM
		 RAM_CS,		// to SDRAM
		 RAM_RAS,		// to SDRAM
		 RAM_CAS,		// to SDRAM
		 RAM_WE,		// to SDRAM
		 RAM_BA,		// to SDRAM
		 RAM_A,		// to SDRAM
		 CLK_IN, 		//from FPGA
		 RESET_IN,		//from FPGA
		 DEBUG2, 		// to pins probe
		 enable_lfsr_wr,	// to lfsr_wr
		 enable_lfsr_rd,	// to lfsr_rd
		 wr_fifow,		// to fifow
		 //d_in_fifow,	// to fifow
		 rd_fifow,		// to fifow
		 wr_fifor,		//to fifor
		 //d_in_fifor,	//to fifor
		 rd_fifor,		//to fifor
		 time_sel,		//from dip-switches
		 emptyr,		//from FIFOr
		 end_rd
		 );
   
  // input [31:0] RAM_D_IN;
   //output [31:0] RAM_D_OUT;
   output 	 RAM_D_EN;   
   output 	 RAM_CLK;
   output 	 RAM_CLKE;
   output 	 RAM_DQMH;
   output 	 RAM_DQML;
   output 	 RAM_CS;
   output 	 RAM_RAS;
   output 	 RAM_CAS;
   output 	 RAM_WE;
   output [1:0]  RAM_BA;
   output [11:0] RAM_A;
   input 	 CLK_IN;
   input 	 RESET_IN;
   inout [31:0]  DEBUG2;
   
   output	 enable_lfsr_wr;	// to lfsr_wr
   output	 enable_lfsr_rd;	// to lfsr_rd
   output	 wr_fifow;		// to fifow
   //output[31:0]	 d_in_fifow;	// to fifow
   output	 rd_fifow;		// to fifow
   output	 wr_fifor;		//to fifor
   //output[31:0]	 d_in_fifor;	//to fifor
   output	 rd_fifor;		//to fifor
   input[15:0]	 time_sel;		//from dip-sw
   input		emptyr;
   output		end_rd;
  
   // counter for states
   wire [15:0] 	   count;
   reg 		   count_en;
   reg 		   count_rst;
  
   // current and nextstate
   reg [3:0] 	 CS;
   reg [3:0] 	 NS;
   
   // different commands that can be issued to the SDRAM
   // for explaination of commands and translation to control 
   // lines refer to "Commands" section,  p12 of SDRAM data sheet
   reg 		 NOP;
   reg 		 loadModeRegister;
   reg 		 active_read, active_write;
   wire 		active;
   reg 		 read;
   reg 		 write;
   reg 		 precharge;
   reg 		 autoPrecharge;
   reg 		 burstTerminate;
   reg 		 autoRefresh;
   reg 		 selfRefresh;

   // counter for incrementing address
   reg [11:0] 	addr_out;
   reg [1:0] 	bank_addr;
   wire [21:0] 	addr_w;
   wire [20:0]	addr_r;
   reg		addr_r_rst, addr_w_rst;
   reg 		addr_r_next, addr_w_next;


  //new signals
 
   reg [2:0] 	write_vec;
   reg 		write4;
   reg [2:0] 	read_vec;
   reg 		read4, read4_i;
   wire		pre_read4;
   reg		pre_write; 
   wire 		pre_write4;
  // reg		time_next_2;

//signals for lfsr
   reg		start_lfsr_w_pulse;
   wire		enable_lfsr_wr, enable_lfsr_rd;
   reg		start_lfsr_wr, start_lfsr_rd;

//signals for fifo
   reg		wr_fifow, start_rd;
   wire		rd_fifor;
   wire 		rd_fifow, wr_fifor;
   reg		end_wr, end_rd;
   wire [29:0] 	time_out;
   wire [15:0] 	time_sel;
   //reg [31:0] 	time_parameter;
   wire [30:0] 	time_add;
   wire		time_next_long;
   reg 		time_next;
   wire 		clk4;
   wire [1:0]	count4; //change from  2 to 3bits

   // mapping of commands to control lines Table1 p12 DATA sheet
   assign 	 RAM_RAS = ~(active | precharge |
			     autoRefresh | selfRefresh
			     | loadModeRegister);
   assign 	 RAM_CAS = ~(read | write |
			     autoRefresh | selfRefresh
			     | loadModeRegister);
   assign 	 RAM_WE = ~(write | burstTerminate |
			    precharge | loadModeRegister);

   //assign 	 RAM_D_OUT = {addr[15:0], addr[15:0]};
   //assign 	 RAM_D_OUT = 32'hf00ff00f; //for test bench write a constant
   assign 	 RAM_D_EN = write4;
   assign 	 RAM_CLK = ~CLK_IN;
   assign 	 RAM_CLKE = 1;
   assign 	 RAM_DQMH = ~(write4 | pre_read4);
   assign 	 RAM_DQML = ~(write4 | pre_read4);
   assign 	 RAM_CS = 0;
   assign 	 RAM_BA = bank_addr;
   assign 	 RAM_A = addr_out;

   assign active = active_write | active_read;

   // muxing of row addr, column addr, and mode-register data
   // into to 11bit address line for 
   always @(addr_w or addr_r or loadModeRegister or read or write or active_read or active_write) 
	begin
      if (write) //(addr <= column addr) when reading and writting, A10=0 (no auto precharge)
			addr_out <= {1'b0, /*`auto_precharge*/ 1'b1, 1'b0, addr_w[6:0], /*0-1-2-3 order */ 2'b00};
      else if (read) //(addr <= column addr) when reading and writting, A10=0 (no auto precharge)
			addr_out <= {1'b0, /*`auto_precharge*/ 1'b1, 1'b0, addr_r[6:0], /*0-1-2-3 order */ 2'b00};
      else if (loadModeRegister) // mode data
			addr_out <= 12'b000000100010; // burst mode 4x, CAS latency 2
      else if (active_write) // command, (addr <= row addr)
			addr_out <= addr_w[20:9];
		else //if (active_read)
			addr_out <= addr_r[20:9];
//		else
//			addr_out <= addr_out;
   end
   
   always@(addr_w or addr_r or active_read or active_write or read or write)
   begin
   	if (active_read|read)
   		bank_addr <= addr_r[8:7];
   	else 
   		bank_addr <= addr_w[8:7];
   end
   		
   
   counter #(16) state_counter(.OUT(count), .CLK(CLK_IN),
			       .ENABLE(count_en), .RESET(count_rst));
   
//this address when writing
//addr_w: 20:9 , 8:7 , 6:0 | row add , bank , column
   counter #(22) addr_w_counter(.OUT(addr_w), .CLK(CLK_IN),
			      .ENABLE(addr_w_next), .RESET(addr_w_rst));

//this address when reading
   counter #(21) addr_r_counter(.OUT(addr_r), .CLK(CLK_IN),
			      .ENABLE(addr_r_next), .RESET(addr_r_rst));



   // State machine, states
   // refer to datasheet for operation of SDRAM

   // This state machine implements a simple data read and write
   // with burst (burst length = 4)
   // after initialization, the statemachine loops between write 
   // and pause modes. When start_rd is given, it loops between 
   // either read and write modes or, when end_wr is flagged, 
   // between read and pause modes.
   // write state: writes into all location in memory 
   //             the last 16bits of the address of the location
   // read state: reads data stored in all memory loaction and
   //             verifies that it matches what was written
   //             in write state. 

   parameter [3:0] idle = 0,
		   init_wait = 1,
		   init_precharge = 2,
		   init_auto_refresh1 = 3,
		   init_auto_refresh2 = 4,
		   init_load_mode_reg = 5,
		   pause_state=7,
		   read_state = 8,
		   write_state = 9,
		   NOP_state = 10,
		   stop_state = 11;
   
   always @ (posedge CLK_IN)
     if (RESET_IN)
       CS <= idle;
     else
       CS <= NS;

   always @(/*AUTOSENSE*/CS or count or start_rd or end_rd or end_wr)
     begin 
	NOP <= 0;
	loadModeRegister <= 0;
	active_read <= 0;
	active_write <= 0;
	read <= 0;
	write <= 0;
	precharge <= 0;
	autoPrecharge <= 0;
	burstTerminate <= 0;
	autoRefresh <= 0;
	selfRefresh <= 0;
	//latch_data <= 0;
	//error_next <= 0;

	count_rst <= 0;	
	count_en <= 1;
	addr_r_rst <= 0;
	addr_w_rst <= 0;

	start_lfsr_w_pulse <= 0;
	pre_write <= 0;
	addr_w_next <= 0;
	addr_r_next <= 0;

	
	case (CS) /* synthesis full_case */

	  idle:begin
	     NS <= init_wait;
	     count_rst <= 1;
	     addr_w_rst <= 1;
	     addr_r_rst <= 1;
	  end

	  init_wait:begin // nop for 100 micro sec
	     NOP <= 1;
	     if (count == 2500/*`init_cycles*/) begin
		NS <= init_precharge;
		count_rst <= 1;
	     end
	     else
	       NS <= init_wait;
	  end

	  init_precharge:begin // 2 cycles (all banks): precharge, nop
	     if (count == 0) begin
		precharge <= 1;
		NS <= init_precharge;
	     end
	     else begin
		NOP <= 1;
		count_rst <= 1;
		NS <= init_auto_refresh1;
	     end
	  end

	  init_auto_refresh1:begin // 8 cycles: auto-refresh, nopX7
	     if (count == 0) begin
		autoRefresh <= 1;
		NS <= init_auto_refresh1;		
	     end
	     else if (count == 7) begin
		NOP <= 1;
		NS <= init_auto_refresh2;
		count_rst <= 1;
	     end
	     else begin
		NOP <= 1;
		NS <= init_auto_refresh1;
	     end
	  end

	  init_auto_refresh2:begin // 8 cycles: auto-refresh, nopX7
	     if (count == 0) begin
		autoRefresh <= 1;
		NS <= init_auto_refresh2;		
	     end
	     else if (count == 7) begin
		NOP <= 1;
		NS <= init_load_mode_reg;
		count_rst <= 1;
	     end
	     else begin
		NOP <= 1;
		NS <= init_auto_refresh2;
	     end
	  end

	  init_load_mode_reg:begin // 2 cycles (addr bus code)
	     if (count == 0) begin
		loadModeRegister <= 1;
		NS <= init_load_mode_reg;
	     end
	     else begin
		NOP <= 1;
		count_rst <= 1;
		NS <= pause_state;		
	     end
	  end

	  pause_state:
	  	begin // 64 cycles (start_lfsr_w NOPx62 pre_write)
	     		if (count == 0) 
		 		begin
		 			start_lfsr_w_pulse <= 1;
					NOP <= 1;
					NS <= pause_state;		
		 		end
	     		else if (count == 8'h3f) //wait for 64 clocks to write 16 words in FIFO_W
		 		begin 
					count_rst <= 1;
		 			NS <= write_state;
	     	 		end
	     		else 
				begin
					NOP <= 1;
					NS <= pause_state;		
	     			end
	  	end

	  write_state:
		begin // 8 cycles: active, nop, write, nop, nop, nop, nop, nop
	     		NS <= write_state;
	     		if (count == 0)
	       		active_write <= 1;
	     		else if (count == 1)
				begin
					pre_write <= 1;
					NOP <= 1;
				end
	     		else if (count == 2)
	       		write <= 1;
	     		else if (count == 7) //7
				begin
					count_rst <= 1;
					NOP <= 1;
					addr_w_next <= 1;
					begin
					if (start_rd == 1)
						NS <= read_state;
					else
						NS <= NOP_state;
					end
	     			end
	     		else
	      		NOP <= 1; 
	  	end

	  NOP_state:
		begin //8 cycles: nop x 8
	     		if (count == 7)//7
				begin
					NOP <= 1;
					count_rst <= 1;
					begin
					if (start_rd == 1)
						NS <= read_state;
					else if (end_wr == 1) 
						NS <= NOP_state;
					else 
						NS <= write_state;
					end
				end
	     		else 
				begin
				NOP <= 1;
				NS <= NOP_state;	
				end	
	     	end


	  read_state:
		begin // 8 cycles: active, nop, read, nop, data, data, data, data
	     		NS <= read_state;
	     		if (count == 0)
	       		active_read <= 1;
	     		else if (count == 2)
	       		read <= 1;
	     		//else if (count == 4|count == 5|count == 6)
	       		//begin
	       			//latch_data <= 1; //shift 4 words into 4 shift registers
	       		//	NOP <= 1;		//or load into FIFO_R
	       		//end
	     		else if (count == 7)//7 
		   		begin
		   			//latch_data <= 1; 
		   			count_rst <= 1;
		   			NOP <= 1;
		   				//if (addr == 21'h000001) //for testing 2 bursts
					if (end_rd == 1)
		     				begin
							NS <= stop_state; 
		     					count_rst <= 1;
		     				end
		   			else if (end_wr == 1)
		     				begin
		     					NS <= NOP_state;
		     					addr_r_next <= 1;
		     				end
					else
		     				begin
		     					NS <= write_state;
		     					addr_r_next <= 1;
		     				end
	         		end
	     		else
	       		NOP <= 1;
	     	end

	  stop_state:
		begin // stop 
			NOP <= 1;
			count_rst <= 1;
			NS <= stop_state;		
	     	end
	endcase
     end 

//////////////////////////////////////
//generate end_wr, end_rd, start_rd
   always@(posedge CLK_IN)
   begin
	if (RESET_IN)
		end_wr <= 0;
   	else if (addr_w == 22'h200000) 
		end_wr <= 1;
   end

   always@(posedge CLK_IN)
   begin
	if (RESET_IN)
		end_rd <= 0;
   	else if (addr_r == 21'h1fffff) 
		end_rd <= 1;
   end

//timer counter for programmable delay
  counter #(30) time_counter(.OUT(time_out), .CLK(CLK_IN),
			      .ENABLE(time_next), .RESET(RESET_IN));

  assign time_next_long = (addr_w[21] == 1) ? 1'b1:1'b0;

  always@(posedge CLK_IN)
	begin
     if (RESET_IN)
       time_next <= 0;
     else if (time_next_long == 1)
       time_next <= 1;
	end

   always @(posedge CLK_IN)
   begin
	if (RESET_IN)
		start_rd <= 0;
   	else if (time_sel == time_out[29:14]) 
		start_rd <= 1;
	else if (end_rd == 1)
		start_rd <= 0;
   end


////////////////////////////////////////////////////////////////////
//generate control signals for the pn_generator: enable_lfsr_wr and enable_lfsr_rd
//start_lfrs_rd is a delayed version of start_rd timed to 
//exactly match the first data read from FIFO with the first data from LFSR_R


   counter #(2) clk4_counter(.OUT(count4), .CLK(CLK_IN),
			       .ENABLE(1'b1), .RESET(RESET_IN));

   assign clk4 = (count4==2'b11)? 1'b1:1'b0; //change to 3 bits




/////////////////////////////////////////////////////////////////////
//generate signals for FIFOs: rd_fifow, wr_fifor,   wr_fifow, rd_fifor 
//create pre_write4 that last 4 clocks for rd_fifow and write4 for RAM_E_EN
//generate start_lfsr_w, start_lfsr_r, clk4

//generate start_lfsr_wr from start_lfsr_pulse and end_wr

   always @ (posedge CLK_IN)
	begin
     		if (RESET_IN | end_wr)
       		start_lfsr_wr <= 0;
     		else if (start_lfsr_w_pulse == 1)
       		start_lfsr_wr <= 1;
	end

  assign enable_lfsr_wr = clk4 & start_lfsr_wr; 
  always@(posedge CLK_IN)
  begin
	if (RESET_IN)
		wr_fifow <= 0;
	else 
		wr_fifow <= enable_lfsr_wr;
  end

//generate start_lfsr_rd, enable_lfsr_rd and rd_fifo_rd 

  always@(posedge CLK_IN)
  begin
	if (RESET_IN)
		start_lfsr_rd <= 0;
	else if (emptyr == 0)
		start_lfsr_rd <= 1;
	else 
		start_lfsr_rd <= 0;
  end

  assign enable_lfsr_rd = clk4 & start_lfsr_rd; 
  assign rd_fifor = enable_lfsr_rd;

//generate rd_fifow

   always @ (posedge CLK_IN)
     if (RESET_IN)
       write_vec <= 3'b000;
     else
       write_vec <= {write_vec[1:0], pre_write};

   always @ (posedge CLK_IN)
     if (RESET_IN)
       write4 <= 1'b0;
     else
       write4 <= pre_write4;

  assign pre_write4 = |write_vec|pre_write;
  assign rd_fifow = pre_write4;

//create read4 that last 4 clocks for wr_fifor 

   always @ (posedge CLK_IN)
     if (RESET_IN)
       read_vec <= 3'b000;
     else
       read_vec <= {read_vec[1:0], read};

   always @ (posedge CLK_IN)
     if (RESET_IN)
	begin
       read4_i <= 1'b0;
	 read4 <= 1'b0;
	end
     else
	begin
       read4_i <= pre_read4;
	 read4 <= read4_i;
	end

  assign pre_read4 = |read_vec|read;
  assign wr_fifor = read4;

  //assign DEBUG2[14:0] = {read, write, active, start_rd, end_wr, end_rd, clk4, CS, NS};
  assign DEBUG2[3:0] = {RESET_IN, read, write, active};
endmodule
