module sdram_top (	 RAM_D_IN, 	// to SDRAM
		 RAM_D_OUT,		// to SDRAM
		 RAM_D_EN,		// to SDRAM
		 RAM_CLK,		// to SDRAM
		 RAM_CLKE,		// to SDRAM
		 RAM_DQMH,		// to SDRAM
		 RAM_DQML,		// to SDRAM
		 RAM_CS,		// to SDRAM
		 RAM_RAS,		// to SDRAM
		 RAM_CAS,		// to SDRAM
		 RAM_WE,		// to SDRAM
		 RAM_BA,		// to SDRAM
		 RAM_A,		// to SDRAM
		 CLK_IN, 		//from FPGA
		 RESET_IN,		//from FPGA
		 DEBUG, 		// to pins probe
		 time_sel,		//from dip-switches
		error_cnt
		//emptyr //for testbench
		 );
	
   input [31:0] RAM_D_IN;
   output [31:0] RAM_D_OUT;
   output 	 RAM_D_EN;   
   output 	 RAM_CLK;
   output 	 RAM_CLKE;
   output 	 RAM_DQMH;
   output 	 RAM_DQML;
   output 	 RAM_CS;
   output 	 RAM_RAS;
   output 	 RAM_CAS;
   output 	 RAM_WE;
   output [1:0]  RAM_BA;
   output [11:0] RAM_A;
   input 	 CLK_IN;
   input 	 RESET_IN;
   inout [31:0]  DEBUG;
   input [15:0] time_sel;
   output[22:0] 	 error_cnt;

   //output	emptyr; //for test bench

  // signals in sdram_cntrl
   wire 	 enable_lfsr_wr;	// to lfsr_wr
   wire 	 enable_lfsr_rd;	// to lfsr_rd
   wire 	 wr_fifow;		// to fifow
   wire [31:0]	 d_in_fifow;	// to fifow
   wire 	 rd_fifow;		// to fifow
   wire 	 wr_fifor;		//to fifor
   wire [31:0]	 d_in_fifor;	//to fifor
   wire 	 rd_fifor;		//to fifor
   wire [15:0]	 time_sel;		//from dip-sw
   wire 		emptyr;


  // signals in pn_generators
  wire clk, reset;
  wire [31:0]  pn_out_r;

  //signals in FIFOs
   wire 		fullw, emptyw;
   wire 		fullr;   
   wire [31:0] d_out_fifor;
   wire		sinit;

  // 32 bit data register for latching sdram data
   wire [31:0]	 data_register1;
   wire [31:0]	 data_register2;
   wire [31:0]	 data_register3;
   wire [31:0]	 data_register4;

 // counters the number of errors
   wire [22:0] 	error_cnt;
   reg 	 	error_next, error_addr_en, error_addr_en2;
   wire		error;
   reg 		error_flag;
   wire [20:0] 	error_addr, addr_fifor;
   wire		error_addr_en_pulse; 
   wire [31:0] 	DEBUG2;

   assign clk = CLK_IN;
   assign reset = RESET_IN;
		 
  sdram_cntrl sdram_cntrl_inst(	 //RAM_D_IN, 	// to SDRAM
		 //RAM_D_OUT,		// to SDRAM
		 RAM_D_EN,		// to SDRAM
		 RAM_CLK,		// to SDRAM
		 RAM_CLKE,		// to SDRAM
		 RAM_DQMH,		// to SDRAM
		 RAM_DQML,		// to SDRAM
		 RAM_CS,		// to SDRAM
		 RAM_RAS,		// to SDRAM
		 RAM_CAS,		// to SDRAM
		 RAM_WE,		// to SDRAM
		 RAM_BA,		// to SDRAM
		 RAM_A,		// to SDRAM
		 CLK_IN, 		//from FPGA
		 RESET_IN,		//from FPGA
		 DEBUG2, 		// to pins probe
		 enable_lfsr_wr,	// to lfsr_wr
		 enable_lfsr_rd,	// to lfsr_rd

		 wr_fifow,		// to fifow
		 //d_in_fifow,	// from fifow output
		 rd_fifow,		// to fifow

		 wr_fifor,		//to fifor
		 //d_in_fifor,	//to fifor input
		 rd_fifor,		//to fifor
		 time_sel,		//from dip-switches
		 emptyr,
		 end_rd
		 );
		 
  pn_gen pn_gen_w(.clk(clk), .reset(reset), .pn_out(d_in_fifow), .pn_shift(enable_lfsr_wr));
  pn_gen pn_gen_r(.clk(clk), .reset(reset), .pn_out(pn_out_r), .pn_shift(enable_lfsr_rd));

  //fifoctlr_cc fifow(.clock_in(clk), .read_enable_in(rd_fifow), .write_enable_in(wr_fifow), 
 //                   .write_data_in(d_in_fifow), .fifo_gsr_in(fifow_gsr), .read_data_out(RAM_D_OUT), 
 //                   .full_out(fullw), .empty_out(emptyw), .fifocount_out(fifow_cnt) );

 // fifoctlr_cc fifor(.clock_in(clk), .read_enable_in(rd_fifor), .write_enable_in(wr_fifor), 
 //                   .write_data_in(d_in_fifor), .fifo_gsr_in(fifor_gsr), .read_data_out(d_out_fifor), 
 //                   .full_out(fullr), .empty_out(emptyr), .fifocount_out(fifor_cnt) );

fifo_sync fifow(.clk(clk), .sinit(sinit), .din(d_in_fifow), .wr_en(wr_fifow), 
			.rd_en(rd_fifow), .dout(RAM_D_OUT), .full(fullw), .empty(emptyw)); 
   
fifo_sync fifor(.clk(clk), .sinit(sinit), .din(d_in_fifor), .wr_en(wr_fifor), 
			.rd_en(rd_fifor), .dout(d_out_fifor), .full(fullr), .empty(emptyr));    

assign sinit = reset;

//latch the data_in from RAM with falling edge of clock, 
//then save it to the FIFO at the next rising edge
   register #(32) ram_data_in(.I(RAM_D_IN), .O(d_in_fifor), .E(wr_fifor), .C(~clk), .R(reset));

   //latch the incoming data for checking
   register #(32) data_reg1(.I(d_in_fifor), .O(data_register1), .E(wr_fifor), .C(clk), .R(RESET_IN));
   register #(32) data_reg2(.I(data_register1), .O(data_register2), .E(wr_fifor), .C(clk), .R(RESET_IN));
   register #(32) data_reg3(.I(data_register2), .O(data_register3), .E(wr_fifor), .C(clk), .R(RESET_IN));
   register #(32) data_reg4(.I(data_register3), .O(data_register4), .E(wr_fifor), .C(clk), .R(RESET_IN));

  //error detection
   //Increments error counter
   //when a inconsistency is encountered. Address of 
   //first error data is recorded. This address and 
   //the final error count is displayed

   counter #(23) error_counter(.OUT(error_cnt), .CLK(clk),
			       .ENABLE(error_next), .RESET(reset));  
   register #(21) error_add_reg(.I(addr_fifor), .O(error_addr), .E(error_addr_en_pulse), .C(clk), .R(reset));
   
   counter #(21) addr_r_counter2(.OUT(addr_fifor), .CLK(clk),
			      .ENABLE(rd_fifor), .RESET(reset));

   always @ (posedge clk)
   begin
     if (reset)
       error_flag <= 0;
     else
       error_flag <= enable_lfsr_rd;
   end

   always @ (posedge clk)
   begin
     if (reset)
       	error_next <= 0;
    else if (end_rd == 1)
       	error_next <= 0;
     else if (error_flag & error)
       	error_next <= 1;
     else
       	error_next <= 0;
   end

   always @ (posedge clk)
   begin
     if (reset)
       	error_addr_en <= 0;
     else if (error_flag & error)
       	error_addr_en <= 1;
   end

   always @ (posedge clk)
   begin
     if (reset)
       	error_addr_en2 <= 0;
     else 
       	error_addr_en2 <= error_addr_en;
   end

 assign error_addr_en_pulse = ~error_addr_en2 & error_addr_en;

 assign error = (pn_out_r == d_out_fifor)? 0:1;


   // debuging code
   //assign 	 DEBUG[9:0] = {RAM_CLK, (CS == write_state), error_cnt[7:0]}; 
   
   //assign 	 DEBUG[15:0] = {data_register4[14:0], RAM_CLK}; 
   //assign 	 DEBUG[31:16] = {data_register4[30:15]}; 
   //assign 	 DEBUG[31:0]  = {d_in_fifor[12:0], wr_fifor, rd_fifow,wr_fifow, rd_fifor, DEBUG2[14:0]};
   assign 	 DEBUG[25:0]  = {error, error_addr,DEBUG2[3:0]};

   //assign 	 DEBUG[15:0] = {error_cnt};
	//assign 	 DEBUG[15:0] = {RAM_D_IN};
	//assign 	 DEBUG[15:0] = {RAM_D_OUT};

   
endmodule   